/*
 * $Header: /home/cvs/jakarta-slide/src/share/org/apache/slide/util/Configuration.java,v 1.4 2000/09/26 02:44:28 remm Exp $
 * $Revision: 1.4 $
 * $Date: 2000/09/26 02:44:28 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.util;

import java.util.Properties;
import java.io.OutputStream;
import java.io.Writer;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import org.apache.slide.common.Domain;
import org.apache.slide.util.Messages;

/**
 * Provides default configuration for Slide components from the
 * <tt>slide.properties</tt> configuration file. All slide features
 * rely on the central configuration file.
 * <p>
 * The configuration file is loaded from the Java <tt>lib</tt>
 * directory, the classpath and the Castor JAR. Properties set in the
 * classpath file takes precedence over properties set in the Java library
 * configuration file and properties set in the Slide JAR, allowing for
 * each customization. All three files are named <tt>slide.properties</tt>.
 * <p>
 * For example, to change the domain init file in use, create a new 
 * configuration file in the current directory, instead of modifying 
 * the global one.
 * 
 * @author <a href="mailto:kvisco@ziplink.net">Keith Visco</a>
 * @author <a href="mailto:arkin@intalio.com">Assaf Arkin</a>
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.4 $ $Date: 2000/09/26 02:44:28 $
 */
public abstract class Configuration {
    
    
    // --------------------------------------------------- Property Inner Class
    
    
    /**
     * Names of properties used in the configuration file.
     */
    public static class Property {
        
        
        // ---------------------------------------------------------- Constants
        
        
        /**
         * Property specifying the domain XML initialization file to use.
         * <pre>
         * org.apache.slide.domain
         * </pre>
         */
        public static final String DomainInitFilename = 
            "org.apache.slide.domain";
        
        
        /**
         * Property specifying that internal security checking is enabled.
         * <pre>
         * org.apache.slide.security
         * </pre>
         */
        public static final String IntegratedSecurity = 
            "org.apache.slide.security";
        
        
        /**
         * Property specifying that internal locking checks are enabled.
         * <pre>
         * org.apache.slide.lock
         * </pre>
         */
        public static final String IntegratedLocking = "org.apache.slide.lock";
        
        
        /**
         * Property specifying whether to run in debug mode.
         * <pre>
         * org.apache.slide.debug
         * </pre>
         */
        public static final String Debug = "org.apache.slide.debug";
        
        
        /**
         * The name of the configuration file.
         * <pre>
         * slide.properties
         * </pre>
         */
        public static final String FileName = "slide.properties";
        
        
        static final String ResourceName = 
            "/org/apache/slide/slide.properties";
        
    }
    
    
    // -------------------------------------------------------------- Variables
    
    
    /**
     * The default properties loaded from the configuration file.
     */
    private static Properties _default;
    
    
    /**
     * True if the default configuration specified debugging.
     */
    private static boolean _debug;
    
    
    /**
     * True if integrated security checks are performed.
     */
    private static boolean _security;
    
    
    /**
     * True if integrated locking is used.
     */
    private static boolean _locking;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Returns true if the default configuration specified debugging.
     */
    public static boolean debug() {
        return _debug;
    }
    
    
    /**
     * Returns true if integrated security checking is used.
     */
    public static boolean useIntegratedSecurity() {
        return _security;
    }
    
    
    /**
     * Returns true if integrated lock checking is used.
     */
    public static boolean useIntegratedLocking() {
        return _locking;
    }
    
    
    // ------------------------------------------------------------ Initializer
    
    
    static {
        getDefault();
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Returns the default configuration file. Changes to the returned
     * properties set will affect all Castor functions relying on the
     * default configuration.
     *
     * @return The default configuration
     */
    public static synchronized Properties getDefault() {
        if (_default == null) {
            load();
        }
        return _default;
    }
    
    
    // ------------------------------------------------------ Protected Methods
    
    
    /**
     * Called by {@link #getDefault} to load the configuration the
     * first time. Will not complain about inability to load
     * configuration file from one of the default directories, but if
     * it cannot find the JAR's configuration file, will throw a
     * run time exception.
     */
    protected static void load() {
        
        File file;
        InputStream is;
        
        // Get detault configuration from the Slide JAR.
        _default = new Properties();
        try {
            _default.load(Configuration.class.getResourceAsStream
                          (Property.ResourceName));
        } catch (Exception except) {
            // This should never happen
            //throw new RuntimeException
            //    (Messages.format("conf.notDefaultConfigurationFile",
            //                     Property.FileName));
        }
        
        // Get overriding configuration from the Java
        // library directory, ignore if not found.
        file = new File(System.getProperty("java.home"), "lib");
        file = new File(file, Property.FileName);
        if (file.exists()) {
            _default = new Properties(_default);
            try {
                _default.load(new FileInputStream(file));
                Domain.info("Configuration found in java.home");
            } catch (IOException except) {
                // Do nothing
            }
        }
        
        // Get overriding configuration from the classpath,
        // ignore if not found.
        try {
            is = Configuration.class.getResourceAsStream("/" 
                                                         + Property.FileName);
            if ( is != null ) {
                _default = new Properties(_default);
                _default.load(is);
                Domain.info("Configuration found in classpath");
            }
        } catch (Exception except) {
            // Do nothing
        }
        
        String prop;
        
        prop = _default.getProperty(Property.Debug, "false");
        if (prop.equalsIgnoreCase("true") || prop.equalsIgnoreCase("on")) {
            _debug = true;
        }
        
        prop = _default.getProperty(Property.IntegratedSecurity, "true");
        if (prop.equalsIgnoreCase("true") || prop.equalsIgnoreCase("on")) {
            _security = true;
        } else {
            _security = false;
        }
        
        prop = _default.getProperty(Property.IntegratedLocking, "true");
        if (prop.equalsIgnoreCase("true") || prop.equalsIgnoreCase("on")) {
            _locking = true;
        } else {
            _locking = false;
        }
        
    }
    
}
