/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/server/org/apache/slide/webdav/method/WebdavMethod.java,v 1.35 2001/08/31 22:02:11 dirkv Exp $
 * $Revision: 1.35 $
 * $Date: 2001/08/31 22:02:11 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.webdav.method;

import java.security.Principal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.io.*;
import java.util.*;
import java.net.URLDecoder;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.apache.util.URLUtil;
import org.apache.util.MD5Encoder;
import org.apache.util.WebdavStatus;
import org.apache.slide.common.*;
import org.apache.slide.authenticate.*;
import org.apache.slide.structure.*;
import org.apache.slide.content.*;
import org.apache.slide.lock.*;
import org.apache.slide.macro.*;
import org.apache.slide.security.*;
import org.apache.slide.webdav.*;

import org.apache.slide.util.logger.Logger;

/**
 * WebDAV method.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @author Christopher Lenz
 */
public abstract class WebdavMethod {
    
    
    // -------------------------------------------------------------- Constants
    
    private static final String LOG_CHANNEL = WebdavMethod.class.getName();
    
    public static final String LOCK_TOKEN = "opaquelocktoken:";
    public static final String SLIDE_NAMESPACE =
        "http://jakarta.apache.org/slide/";
    public static final String SLIDE_NAMESPACE_ABBREV = "S";
    
//  public static final String PRINCIPAL =
//      "org.apache.slide.webdav.method.principal";
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Requested Uri.
     */
    protected String requestUri;
    
    
    /**
     * Servlet request.
     */
    protected HttpServletRequest req;
    
    
    /**
     * Servlet response.
     */
    protected HttpServletResponse resp;
    
    
    /**
     * Configuration.
     */
    private WebdavServletConfig config;
    
    
    /**
     * Request body.
     */
    protected String requestBody;
    
    
    /**
     * Namespace access token.
     */
    protected NamespaceAccessToken token;
    
    
    /**
     * Structure helper.
     */
    protected Structure structure;
    
    
    /**
     * Content helper.
     */
    protected Content content;
    
    
    /**
     * Security helper.
     */
    protected Security security;
    
    
    /**
     * Lock helper.
     */
    protected Lock lock;
    
    
    /**
     * Macro helper.
     */
    protected Macro macro;
    
    
    /**
     * Slide token.
     */
    protected SlideToken slideToken;
    
    
    /**
     * MD5 message digest provider.
     */
    protected static MessageDigest md5Helper;


    /**
     * The MD5 helper object for this class.
     */
    protected static final MD5Encoder md5Encoder = new MD5Encoder();
    
    
    // -------------------------------------------------- Static Initialization


    static {

        // Load the MD5 helper used to calculate signatures.
        try {
            md5Helper = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            System.out.println(e.toString());
            throw new IllegalStateException();
        }
    }


    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Method constructor.
     */
    public WebdavMethod(NamespaceAccessToken token, HttpServletRequest req,
                        HttpServletResponse resp, WebdavServletConfig config) {
        
        this.req = req;
        this.resp = resp;
        this.config = config;
        
        this.token = token;
        this.structure = token.getStructureHelper();
        this.content = token.getContentHelper();
        this.security = token.getSecurityHelper();
        this.lock = token.getLockHelper();
        this.macro = token.getMacroHelper();

        this.slideToken = WebdavUtils.getSlideToken(req);
        this.requestUri = WebdavUtils.getRelativePath(req, config);
        
        parseHeaders();
        
    }
    
    
    // --------------------------------------------------------- Public Methods
    
    
    /**
     * Returns the configuration of the WebdavServlet.
     *
     * @return WebdavServletConfig
     */
    public WebdavServletConfig getConfig() {
        
        return config;
    }
    
    
    /**
     * Return an absolute URL (absolute in the HTTP sense) based on a Slide
     * path.
     */
    public String getFullPath(String path) {
        
        if (path.startsWith("/"))
            return WebdavUtils.encodeURL(req.getContextPath() + path);
        else
            return WebdavUtils.encodeURL(req.getContextPath() + "/" + path);
    }

     
    /**
     * Returns a Slide path based on an absolute URL
     * (absolute in the HTTP sense)
     */
    public String getSlidePath(String fullpath) {
        // strip off the protocol://host:port part
        if (fullpath.indexOf("://") >= 0) {
            fullpath=fullpath.substring(fullpath.indexOf("://")+3);
            fullpath=fullpath.substring(fullpath.indexOf("/"));
        }

        // strip off the servlet context path
        String contextPath=req.getContextPath();
        if (fullpath.startsWith(contextPath)) {
            fullpath=fullpath.substring(contextPath.length());
        }
        return fullpath;
    }
    
    
    /**
     * Exceute method.
     *
     * @exception WebdavException
     */
    public void run()
        throws WebdavException {
        
        boolean transactionIsStarted = false;
        
        try {
            parseRequest();
            if (methodNeedsTransactionSupport()) {
                token.begin();
                transactionIsStarted = true;
            }
            executeRequest();
            if (methodNeedsTransactionSupport()) {
                token.commit();
                transactionIsStarted = false;
            }
        } catch (WebdavException ex) {
            // Ignore the WebDav Exception and assume that the response code
            // is already set.
        } catch (Exception ex) {
            Domain.log(ex,LOG_CHANNEL,Logger.ERROR);
            resp.setStatus(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
            throw new WebdavException(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
        } finally {
            if (transactionIsStarted && methodNeedsTransactionSupport()) {
                // Something went wrong, we are here and the TA is still open
                try {
                    token.rollback();
                } catch (Exception e) {
                }
            }
        }
        
    }
    
    
    // ------------------------------------------------------ Protected Methods
    
    
    /**
     * Return true if the method needs transaction support.
     * Return true if the method needs transaction support, e.g.
     * the methods put, proppatch would need TA support. The
     * methods get, propfind would not need transaction support.
     */
    protected boolean methodNeedsTransactionSupport() {
        return true;
        // this method should return false and e.g. PutMethod should
        // overwrite it to true, but PropFind creates a RevisionDescriptors
        // on the fly
    }
    
    
    /**
     * Read request contents.
     *
     * @param req Request object handed out by the servlet container
     * @return char[] Array of char which contains the body of the request
     */
    protected void readRequestContent() {
        
        if (req.getContentLength() == 0)
            return;
        
        // TODO : Modify this and make it chunking aware
        
        try {
            requestBody = new String(NodeRevisionContent.readFromStream(req.getInputStream()),
                                     getEncodingString(req.getCharacterEncoding()));
        }
        catch (Exception e) { 
            Domain.log(e,LOG_CHANNEL,Logger.ERROR);
        }
    }

    
    
    /**
     * Translate the encoding string into a proper Java encoding String.
     */
    public static String getEncodingString(String httpEncoding) {
        String result = httpEncoding;
        if (result == null) result = System.getProperty("file.encoding");
        if (result.startsWith("\"")) result = result.substring(1, result.length());
        if (result.endsWith("\""))   result = result.substring(0, result.length()-1);
        return result;
    }
    
    
    /**
     * Parse lock token headers.
     */
    protected void parseHeaders() {
        
        // Retrieve if header
        String ifHeader = req.getHeader("If");
        //System.out.println("If header : " + ifHeader);
        
        if (ifHeader == null)
            return;
        
        // Parse if header and extract the lock tokens
        int pos = ifHeader.indexOf(LOCK_TOKEN);
        int endPos = -1;
        int offset = LOCK_TOKEN.length();
        String lockToken = null;
        
        while (pos != -1) {
            
            endPos = ifHeader.indexOf('>', pos + offset);
            if (endPos == -1) {
                lockToken = ifHeader;
            } else {
                lockToken = ifHeader.substring(pos + offset, endPos);
            }
            
            //System.out.println("Lock Token found :-" + lockToken + "-");
            slideToken.addLockToken(lockToken);
            
            pos = ifHeader.indexOf(LOCK_TOKEN, endPos);
            
        }
        
    }
    
    
    /**
     * Returns the lock token value from the lock token header.
     *
     * @return String Opaque lock token value
     */
    protected String parseLockToken(String lockTokenValue) {
        int semi = lockTokenValue.indexOf(":");
        String result = null;
        try {
            result = lockTokenValue.substring(semi + 1,
                                              lockTokenValue.length() - 1);
        } catch (Exception e) {
            Domain.log(e,LOG_CHANNEL,Logger.ERROR);
        }
        return result;
    }
    
    /**
     * Test if a resource given by a path is a collection
     */
    protected boolean isCollection(String path) {
        return WebdavUtils.isCollection(token, slideToken, path);
    }
                
    
    /**
     * Parse WebDAV XML query.
     *
     * @exception WebdavException
     */
    protected abstract void parseRequest()
        throws WebdavException, IOException;
    
    
    /**
     * Parse WebDAV XML request body, and return the DOM representation.
     *
     * @exception ParserConfigurationException If the JAXP factory cannot
     *             configure a parser given its current configuration
     *             parameters.
     * @exception SAXException If any parse errors occur.
     * @exception IOException If any IO errors occur.
     */
    protected Document parseRequestContent()
        throws ParserConfigurationException, SAXException, IOException {

        if (requestBody == null) {
            readRequestContent();
            if (requestBody == null) {
                // there is no request content body, so there's nothing to
                // parse
                return null;
            }
        }

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);

        DocumentBuilder builder = factory.newDocumentBuilder();
        return builder.parse(new InputSource(new StringReader(requestBody)));

    }


    /**
     * Generate XML response.
     *
     * @exception WebdavException
     */
    protected abstract void executeRequest()
        throws WebdavException, IOException;
    
    
    /**
     * Simulate MS IIS5 ?
     *
     * @return boolean
     */
    protected boolean isMsProprietarySupport() {
        return (token.getNamespaceConfig().getParameter("ms") != null);
    }
    
    
    // -------------------------------------------------------- Private Methods
    
    
    /**
     * Get return status based on exception type.
     */
    protected int getErrorCode(Exception ex) {
        if ( !(ex instanceof SlideException) ) {
            return WebdavStatus.SC_INTERNAL_SERVER_ERROR;
        } else {
            return getErrorCode((SlideException)ex);
        }
    }
    
    
    /**
     * Get return status based on exception type.
     */
    protected int getErrorCode(SlideException ex) {
        try {
            throw ex;
        } catch(ObjectNotFoundException e) {
            return WebdavStatus.SC_NOT_FOUND;
        } catch(ConflictException e) {
            return WebdavStatus.SC_CONFLICT;
        } catch(ForbiddenException e) {
            return WebdavStatus.SC_FORBIDDEN;
        } catch(AccessDeniedException e) {
            return WebdavStatus.SC_FORBIDDEN;
        } catch(ObjectAlreadyExistsException e) {
            return WebdavStatus.SC_PRECONDITION_FAILED;
        } catch(ServiceAccessException e) {
            return getErrorCode((ServiceAccessException)ex);
        } catch(ObjectLockedException e) {
            return WebdavStatus.SC_LOCKED;
        } catch(SlideException e) {
            return WebdavStatus.SC_INTERNAL_SERVER_ERROR;
        }
    }


    
    /**
     * Get return status based on exception type.
     */
    protected int getErrorCode(ServiceAccessException ex) {
        Exception cause = ex.getCauseException();
        if (cause == null || !(cause instanceof SlideException) )  {
            return WebdavStatus.SC_INTERNAL_SERVER_ERROR;  // this is the default}
        } else  {
            return getErrorCode((SlideException)cause);
        }
    }


}


